package scripts.migration.from4_4To4_5.general;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.NodeData;
import info.magnolia.context.Context;
import info.magnolia.context.MgnlContext;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

import info.magnolia.cms.i18n.Messages;
import info.magnolia.cms.i18n.MessagesManager;
import info.magnolia.cms.util.ContentUtil;

/**
 * The goal of this class is to perform a simple comparison between two nodes, one acting
 * as 'master' and the other one as 'slave' extending the master.
 * The script will determine similarities and eliminate those in the slave. It will also
 * resolve all messages and remove redundant ones and replace hardcoded Strings if the
 * other compared node contains an i18n key for it.
 * The general process is to work bottom up if you need to chain some comparisons. For
 * example: you want an extras teaser (C) to extends its normal counterpart (B), while this
 * is nothing more than an internal link teaser (A) with different labels, you will first have
 * to compare the extras teaser to its counterpart and then the counterpart to the general
 * internal teaser. C > B > A. This is due to the fact that it cannot resolve the extended
 * content at the moment of the comparison.
 *
 */
class TransformExtends{

  Context ctx
  HierarchyManager hm

  Messages msgs
  String modulePath
  String i18nBasename
  Collection transformExtendsLog = []
  Collection masterNodePaths = []
  Collection slaveNodePaths = []
  Map deletedI18nMessages = [:]

  /**
   * Main method which iterates through the comparison map and delegates the
   * key/value pair to {@link compareAndInsertExtends(String, String)}.
   *
   * @param comparisonMap Map containing all master/slave node paths relative to the module path
   * @param moduleName Name of the module that contains the nodes
   * @param i18nBasename i18nBasename that is needed for resolving the i18n keys
   *
   * @return Map(String, String) A map that contains log information about all deleted/replaced i18n keys
   */
  def Map executeComparison(Map comparisonMap, String moduleName, String i18nBasename) {
    hm = ctx.getHierarchyManager('config')
    this.modulePath = "/modules/"+moduleName
    this.i18nBasename = i18nBasename
    msgs = MessagesManager.getMessages(i18nBasename)

    //iterate the comparison map
    comparisonMap.each { slave, master ->
      compareAndInsertExtends(master, slave)
    }
    return deletedI18nMessages
  }

  /**
   * Prints the full log that was generated throughout the comparison process.
   */
  def migrationLog () {
    println transformExtendsLog
  }

  /**
   * Prints out the Map with all log information about the deleted/replaced i18n keys.
   */
  def deletedI18nMessagesLog () {
      println deletedI18nMessages
  }

  /**
   * Returns a map that contains log information about all deleted/replaced i18n keys.
   */
  def Map getDeletedI18nMessages () {
      return deletedI18nMessages
  }

  /**
   * Creates a node called 'inheritable' (with corresponding extends) within the passed node path.
   * Mostly used for extras teaser.
   */
  def private createInheritableNode(String nodePath) {
    if (hm.isExist(nodePath)) {
      def node = hm.getContent(nodePath)
      def inheritableNode = node.createContent("inheritable", ItemType.CONTENTNODE)
      inheritableNode.setNodeData("extends", "/modules/standard-templating-kit/dialogs/generic/controls/inheritable")
      node.save()
    }
  }

 /**
   * Creates an extends/override node with the name nodeName.
   *
   * @param nodePath The path where to create the node in
   * @param nodeName The name of the node that will contain the extends/override property
   */
  def private createExtendsOverride(String nodePath, String nodeName) {
    def node = hm.getContent(nodePath)
    def extendsOverrideNode = node.createContent(nodeName, ItemType.CONTENTNODE)

    //Special case: if parent is called availableComponents we set enabled=false rather than extends=override
    if (node.getName().equalsIgnoreCase("availableComponents")) {
      transformExtendsLog.add("Added enabled-false to the availableComponent in "+nodePath)
      extendsOverrideNode.setNodeData("enabled", false)
    }
    else {
      transformExtendsLog.add("Added extends-override to "+nodePath)
      extendsOverrideNode.setNodeData("extends", "override")
    }
    node.save()
  }

  /**
   * Determines whether the node under the passed path qualifies for an extends/override.
   * This could happen at any point nested deep inside a slave node, therefore we need its
   * base path as well to ensure we're not iterating over a node that has already been
   * processed before.
   *
   * @param basePath The current node's base path, meaning the slave's path
   * @param currentPath The current path. May be deep down in the slave's hierarchy
   *
   * @return boolean
   */
  def private needsExtendsOverride(String basePath, String currentPath) {
    def parentPath = StringUtils.substringBeforeLast(currentPath, "/")
    def parentOfParentPath = StringUtils.substringBeforeLast(parentPath, "/")

    def topNodeHasExtends = hm.isExist(modulePath+basePath+"/extends")
    def parentExists = hm.isExist(parentPath)
    def parentHasExtends = hm.isExist(parentPath + "/extends")
    def parentExtendsIsOverride = false
    if (parentHasExtends) {
      parentExtendsIsOverride = hm.getContent(parentPath).getNodeData("extends").getString().equalsIgnoreCase("override")
    }
    def parentHasEnabled = hm.isExist(parentPath + "/enabled")
    def parentEnabled = true
    if (parentHasEnabled) {
      parentEnabled = hm.getContent(parentPath).getNodeData("enabled").getString().equalsIgnoreCase("true")
    }
    def hasRoles = ("roles").equalsIgnoreCase(StringUtils.substringAfterLast(currentPath, "/"))

    if (topNodeHasExtends) {
      //At the top of the node that we want to iterate over, having an 'extends' means that the script most
      //likely ran already. We don't want to override what we've been able to delete thanks to 'extends'.
      return false
    }
    else if (!parentExists) {
      //You will end up here in case of subnodes of nodes that already have an extends-override, so the subnode
      //was not created in the slave previously.
      return false
    }
    else if (parentHasExtends && parentExtendsIsOverride) {
      //If the parent of the current node has an extends=override already
      return false
    }
    else if (parentHasEnabled && !parentEnabled && hasRoles) {
      //If the parent has enabled=false and is an availableComponent
      return false
    }
    else {
      //We covered all the cases that we DON'T want to extends/override
      return true
    }
  }

  /**
   * Deletes the passed property (Node data).
   */
  def private deleteProperty(NodeData nodeData) {
    def propertyName = nodeData.getName()
    def parentNodePath = nodeData.parent.getHandle()

    nodeData.delete()
    transformExtendsLog.add("deleted: '$propertyName' in $parentNodePath")
  }

  /**
   * Deletes all nodes from the current slave that are empty.
   * This method is set up in a way that it starts from bottom up in order to
   * ensure that a node containing another empty node is also deleted. This method
   * will be executed at the end of each comparison iteration and then the map of
   * slave node paths will be generated on a new iteration.
   */
  def private deleteEmptyContentNodes() {
    //First, we need to sort the map by depth in order to start deleting from the bottom up
    //sorting by length/size wouldn't make sense since <node/someexample> is longer than <node/1/2/3>.
    //the "-" in front is needed for sorting in descending order
    slaveNodePaths = slaveNodePaths.sort { -StringUtils.countMatches(it, "/") }

    def tempNode
    slaveNodePaths.each { nodePath ->
      if (hm.isExist(nodePath)){
        tempNode = hm.getContent(nodePath)
        if (!tempNode.hasChildren() && tempNode.getNodeDataCollection().isEmpty()) {
          tempNode.delete()
        }
      }
    }
  }

  /**
   * This method compares the properties from the slave to the master and resolves
   * possible i18n keys to compare their retrieved values as well.
   *
   * @param master Master node passed as {@link Content}
   * @param slave Slave node passed as {@link Content}
   *
   * @return int Number of changes registered in this method. Used to prevent saving if there were no changes.
   */
  def private comparePropertiesAndDeleteEqual(Content master, Content slave) {
      def tempCount = 0

      slave.getNodeDataCollection().each { NodeData slaveProperty ->
        def slavePropertyName = slaveProperty.getName()
        //continue only if the master has the same property name
        if (master.hasNodeData(slaveProperty.getName())) {
            def masterPropertyValue = master.getNodeData(slavePropertyName).getString()
            def slavePropertyValue = slaveProperty.getString()
            //1) plain text comparison
            if (slavePropertyValue.equalsIgnoreCase(masterPropertyValue)) {
              deleteProperty(slaveProperty)
              tempCount++
            }
            //2) master and slave are i18nized and their resolved values are equal
            else if (msgs.get(slavePropertyValue).equalsIgnoreCase(msgs.get(masterPropertyValue))) {
              deleteProperty(slaveProperty)
              deletedI18nMessages.put(slavePropertyValue, "Value was removed in "+slaveProperty.parent.getHandle())
              tempCount++
            }
            //3) master is i18nized, slave is plain text, but their values are equal
            else if (slavePropertyValue.equalsIgnoreCase(msgs.get(masterPropertyValue))) {
              deleteProperty(slaveProperty)
              deletedI18nMessages.put(slavePropertyValue, "Value was removed in "+slaveProperty.parent.getHandle())
              tempCount++
            }
            //4) master is not i18nized, slave is (!), but their values are equal
            else if (msgs.get(slavePropertyValue).equalsIgnoreCase(masterPropertyValue)) {
              // in this case actually replace the master's property with the i18nized string
              deleteProperty(slaveProperty)
              deletedI18nMessages.put(slavePropertyValue, "Attention, slave property replaced non-i18n master property in: "+master.getHandle())
              master.setNodeData(slavePropertyName, slavePropertyValue)
              master.save()
              tempCount++
            }
        }
      }//collection
      return tempCount
  }

  /**
   * This is the key method that populates master and slave node path maps and then
   * delegates the current paths either to {@link comparePropertiesAndDeleteEqual(String, String)}
   * or {@link createExtendsOverride(String, String)} in case the slave path does not exist and
   * needs to be overridden.
   */
  def private compareAndInsertExtends(String masterBasePath, String slaveBasePath) {
      def changesCount = 0

      def masterBasePathExists = hm.isExist(modulePath+masterBasePath)
      def slaveBasePathExists = hm.isExist(modulePath+slaveBasePath)

      if (masterBasePathExists && slaveBasePathExists) {
        transformExtendsLog.add("Both nodes exist")
        def masterBaseNode = hm.getContent(modulePath+masterBasePath)
        def slaveBaseNode = hm.getContent(modulePath+slaveBasePath)

        changesCount += comparePropertiesAndDeleteEqual(masterBaseNode, slaveBaseNode)

        masterNodePaths = []
        slaveNodePaths = []

        ContentUtil.collectAllChildren(masterBaseNode).each{ Content masterContentNode ->
            def masterContentNodePath = masterContentNode.getHandle()
            masterNodePaths.add(masterContentNodePath)
            slaveNodePaths.add(StringUtils.replaceOnce(masterContentNodePath,masterBasePath,slaveBasePath))
        }

        masterNodePaths.eachWithIndex{ masterPath, index ->
          def slavePath = slaveNodePaths.getAt(index)

          if (hm.isExist(masterPath) && hm.isExist(slavePath)) {
            def tempMasterNode = hm.getContent(masterPath)
            def tempSlaveNode = hm.getContent(slavePath)
            changesCount += comparePropertiesAndDeleteEqual(tempMasterNode, tempSlaveNode)
          }
          else if (hm.isExist(masterPath) && needsExtendsOverride(slaveBasePath, slavePath)) {
            createExtendsOverride(StringUtils.substringBeforeLast(slavePath,"/"), StringUtils.substringAfterLast(slavePath, "/"))
            changesCount ++
          }

          // treat the special cases
          if (slavePath.equalsIgnoreCase(modulePath+"dialogs/components/extras/stkExtrasContact/tabTeaser")) {
            createInheritableNode(slavePath)
            changesCount++
          }
        }

        //finally, insert the 'extends' property, delete all empty nodes and save the changes - if there were any.
        if (changesCount > 0) {
          slaveBaseNode.setNodeData('extends', modulePath+masterBasePath)
          deleteEmptyContentNodes()
          slaveBaseNode.save()
          transformExtendsLog.add("[Count: $changesCount] Successfully applied extends feature to $modulePath$slaveBasePath")
        }
        else {
          transformExtendsLog.add("There was nothing to be changed!")
        }


      }
      else {
        transformExtendsLog.add("Either one or both nodes DO NOT exist, please verify that the paths are correct. Original: $modulePath$masterBasePath- Node that needs to be transformed: $modulePath$slaveBasePath")
      }
  }

  /**
   * Method that does all preliminary preparations for STK. It creates
   * masters (dialogs) and then fills the comparison map that will be passed
   * onto {@link createExtendsOverride(String, String)}. Refer to this setup if
   * you want to use the extends feature for your own modules.
   */
  def static Map prepareAndCreateSTKComparisonMap(Context ctx) {
      Map comparisonMap = [:]
      String modulePath = "/modules/standard-templating-kit/"
      def hm = MgnlContext.getHierarchyManager('config')

      // preparation: create the master configurations for page properties, teasers and teaser groups
      def genericNode = hm.getContent(modulePath+"dialogs/generic")
      def masterNode = genericNode.createContent("master")
      def changeTeaserMap = ["/tabLinkList" : ""]
      genericNode.save()

      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/pages/article/stkArticleProperties", modulePath+"dialogs/generic/master/basePageProperties", [:])
      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/components/teasers/stkTeaser", modulePath+"dialogs/generic/master/baseTeaserInternal", changeTeaserMap)
      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/components/teasers/stkTeaserGroup", modulePath+"dialogs/generic/master/baseTeaserGroup", [:])
      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/components/teasers/stkTeaserNewsList", modulePath+"dialogs/generic/master/baseTeaserList", ["/tabTeaser/hideTeaserImage":""])
      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemExternalPage", modulePath+"dialogs/generic/master/baseTeaserExternal", ["/tabTeaserOverwrite/shortTitle":""])
      MigrationUtil.createMasterDialogThroughCopy(ctx, modulePath+"dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemDownloadFile", modulePath+"dialogs/generic/master/baseTeaserDownload", ["/tabTeaserOverwrite/shortTitle":""])
      genericNode.save()

      // preparation: rename the tabMain tab of teaser groups to tabTeaser
      MigrationUtil.renameNodeIfExisting(hm, modulePath+"dialogs/components/teasers/stkTeaserPaging/tabMain", "tabTeaser")
      MigrationUtil.renameNodeIfExisting(hm, modulePath+"dialogs/components/teasers/stkTeaserCarousel/tabMain", "tabTeaser")
      MigrationUtil.renameNodeIfExisting(hm, modulePath+"dialogs/components/teasers/stkTeaserFingerTabbed/tabMain", "tabTeaser")
      //Typo of node: tabTeaserOverwite missing 'r'
      MigrationUtil.renameNodeIfExisting(hm, modulePath+"dialogs/components/teasers/stkTeaserCarouselItem/tabTeaserOverwite", "tabTeaserOverwrite")

      //1) bottom-up: compare extras to teasers
      comparisonMap.put("dialogs/components/extras/stkExtrasContact","dialogs/components/teasers/stkTeaserContact")
      comparisonMap.put("dialogs/components/extras/stkExtrasBoxGroup","dialogs/components/teasers/stkTeaserGroup")
      comparisonMap.put("dialogs/components/extras/stkExtrasNewsList","dialogs/components/teasers/stkTeaserNewsList")
      comparisonMap.put("dialogs/components/extras/stkExtrasEventsList","dialogs/components/teasers/stkTeaserEventsList")
      comparisonMap.put("dialogs/components/extras/horizontalTabbed/stkExtrasHorizontalTabbed","dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed")
      comparisonMap.put("dialogs/components/extras/stkExtrasInternalPage","dialogs/components/teasers/stkTeaser")
      comparisonMap.put("dialogs/components/extras/stkExtrasPureLinkList","dialogs/components/teasers/stkTeaserPureLinkList")

      //2) bottom-up: compare normal teasers + groups with their masters
          //master: baseTeaser
      comparisonMap.put("dialogs/components/teasers/stkTeaser","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/teasers/stkTeaserOpener","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/teasers/stkTeaserCarouselItem","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/teasers/stkTeaserFingerTabbedItem","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemInternalPage","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/stages/stkStageXL","dialogs/generic/master/baseTeaserInternal")
      comparisonMap.put("dialogs/components/promos/stkPromo","dialogs/generic/master/baseTeaserInternal")
          //master: baseTeaserGroup
      comparisonMap.put("dialogs/components/teasers/stkTeaserGroup","dialogs/generic/master/baseTeaserGroup")
      comparisonMap.put("dialogs/components/teasers/stkTeaserPaging","dialogs/generic/master/baseTeaserGroup")
      comparisonMap.put("dialogs/components/teasers/stkTeaserCarousel","dialogs/generic/master/baseTeaserGroup")
      comparisonMap.put("dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed","dialogs/generic/master/baseTeaserGroup")
          //master: baseTeaserList
      comparisonMap.put("dialogs/components/teasers/stkTeaserNewsList","dialogs/generic/master/baseTeaserList")
      comparisonMap.put("dialogs/components/teasers/stkTeaserEventsList","dialogs/generic/master/baseTeaserList")
          //master: baseTeaserExternal
      comparisonMap.put("dialogs/components/extras/stkExtrasExternalPage","dialogs/generic/master/baseTeaserExternal")
      comparisonMap.put("dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemExternalPage","dialogs/generic/master/baseTeaserExternal")
          //master: baseTeaserDownload
      comparisonMap.put("dialogs/components/extras/stkExtrasDownloadFile","dialogs/generic/master/baseTeaserDownload")
      comparisonMap.put("dialogs/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemDownloadFile","dialogs/generic/master/baseTeaserDownload")

      //3) components
      comparisonMap.put("templates/components/teasers/stkTeaserOpener","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/promos/stkPromo","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/teasers/stkTeaserCarouselItem","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/teasers/stkTeaserFingerTabbedItem","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/teasers/stkTeaserPaging","templates/components/teasers/stkTeaserCarousel")
      comparisonMap.put("templates/components/stages/stkStagePaging","templates/components/teasers/stkTeaserCarousel")
      comparisonMap.put("templates/components/footer/stkFooterLinkInternal","templates/components/links/stkInternalLink")
      comparisonMap.put("templates/components/footer/stkFooterLinkExternal","templates/components/links/stkExternalLink")
          //all estras teaser compared to the main teasers
      comparisonMap.put("templates/components/extras/stkExtrasInternalPage","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabbed","templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed")
      comparisonMap.put("templates/components/extras/stkExtrasContact","templates/components/teasers/stkTeaserContact")
      comparisonMap.put("templates/components/extras/stkExtrasBoxGroup","templates/components/teasers/stkTeaserGroup")
      comparisonMap.put("templates/components/extras/stkExtrasEventsList","templates/components/teasers/stkTeaserEventsList")
      comparisonMap.put("templates/components/extras/stkExtrasNewsList","templates/components/teasers/stkTeaserNewsList")
      comparisonMap.put("templates/components/extras/stkExtrasPureLinkList","templates/components/teasers/stkTeaserPureLinkList")
          //aligning horizontal tabbed itmes with possible 'masters'
      comparisonMap.put("templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabitemInternalPage","templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemInternalPage")
      comparisonMap.put("templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabitemDownloadFile","templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemDownloadFile")
      comparisonMap.put("templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabitemExternalPage","templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemExternalPage")
      comparisonMap.put("templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemInternalPage","templates/components/teasers/stkTeaser")
      comparisonMap.put("templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemDownloadFile","templates/components/extras/stkExtrasDownloadFile")
      comparisonMap.put("templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemExternalPage","templates/components/extras/stkExtrasExternalPage")

      //4) templates
      comparisonMap.put("templates/pages/stkLargeArticle", "templates/pages/stkArticle")  //already down there
      comparisonMap.put("templates/pages/stkNews", "templates/pages/stkArticle")
      comparisonMap.put("templates/pages/stkEvent", "templates/pages/stkArticle")
      comparisonMap.put("templates/pages/stkGlossaryTerm", "templates/pages/stkArticle")
      comparisonMap.put("templates/pages/demo-features/stkSectionNoHeader", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionNoOpener", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionNoExtras", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionFloating", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionWithOpener", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionWithHeader", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionWithTwoExtras", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionNoExtrasPromos", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionWithIntroNoHeader", "templates/pages/stkSection")
      comparisonMap.put("templates/pages/demo-features/stkSectionNoIntroWithHeader", "templates/pages/stkSection")

      //5) page properties
      String basePagePropertiesPath = "dialogs/generic/master/basePageProperties"
      comparisonMap.put("dialogs/pages/faq/stkFAQProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/form/stkFormProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/news/stkNewsProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/event/stkEventProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/article/stkArticleProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/section/stkSectionProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/siteMap/stkSiteMapProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/glossary/stkGlossaryProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/glossary/stkGlossaryTermProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/glossary/stkGlossaryLetterProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/largeArticle/stkLargeArticleProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/imageGallery/stkImageGalleryProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/newsOverview/stkNewsOverviewProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/eventsOverview/stkEventsOverviewProperties", basePagePropertiesPath)
      comparisonMap.put("dialogs/pages/searchResult/stkSearchResultProperties", basePagePropertiesPath)

      return comparisonMap;
    }

}