package scripts.migration.from4_4To4_5.util

import groovy.xml.*
import org.custommonkey.xmlunit.*

/**
* A usage sample for the DiffUtil class. This class will help comparing repositories of different Magnolia instances by producing an XML view of them stripped of
* all unnecessary nodes and properties (you can of course configure them). You need to adapt the following variable values to your needs.
* If comparison by XMLUnit returns non identical (or non similar) files (see  http://xmlunit.sourceforge.net/api/index.html?org/custommonkey/xmlunit/Diff.html)
* you can run a tool to compare them.
* I.e. on a Mac you can run TextWrangler -> Search -> Find Differences...
*/
workspace = 'config'
workspaceAbsPath = '/modules'
outputFolder = '/path/to/folder/'
nodesToIgnore = ['MetaData']
propertiesToIgnore = ['jcr:uuid', 'jcr:created','jcr:createdBy']
expectedXML = 'install.xml'
toCheckXML = 'update.xml'

println  "Starting export of workspace '${workspace}' starting at '${workspaceAbsPath}' to '${outputFolder}'..."
start = System.currentTimeMillis()
DiffUtil.outputXML(workspace, workspaceAbsPath, outputFolder + expectedXML, nodesToIgnore, propertiesToIgnore)
DiffUtil.outputXML(workspace, workspaceAbsPath, outputFolder + toCheckXML, nodesToIgnore, propertiesToIgnore)

XMLUnit.setIgnoreWhitespace(true)

//the first file passed to XML.compareXML(..) is the expected xml. The second one is the xml to be matched against the expected one.
xmlDiff = XMLUnit.compareXML(new File(outputFolder + expectedXML).text, new File(outputFolder + toCheckXML).text)
println  "Export and XML comparison done in ${System.currentTimeMillis() - start} ms"

//Alternativly you can check if files are similar. I.e. xmlDiff.similar()
if(!xmlDiff.identical()) {
   /*
      //you can also ask XMLUnit to print out the diffs although I find the output a bit confusing. Probably needs to be fine tuned with a custom DifferenceListener
       outputDiffs = new File(outputFolder+'output-diffs.txt')
        //recreate the file
        if(outputDiffs.exists()) {
            outputDiffs.delete()
            outputDiffs.createNewFile()
        }
       new DetailedDiff(xmlDiff).allDifferences.each { diff ->
         outputDiffs.append diff.toString() + '\n'
       }
    */
   return "Warning: files are NOT identical!"
} else {
   return "Files are identical."
}


/**
 * DiffUtil class implementation.
 */
@Grapes(
@Grab(group='xmlunit', module='xmlunit', version='1.1')
)
class DiffUtil {
    def static reservedNames = (MarkupBuilder.metaClass.methods.name + MarkupBuilder.metaClass.metaMethods.name).unique()

    /**
     * Traverse a hierarchy of nodes starting from the node passed as root and builds an XML file out of it. Should a node being a reserved groovy name (i.e. 'print') the
     * node node will have an underscore appended to it. This method is used internally by <code>outputXML(..)</code>.
     * @param builder an instance of MarkupBuilder
     * @param node the root node
     * @param attrs a map of attributes which will be applied to the current node. Can be null or empty.
     */
    def static traverse(builder, node, attrs) {

        def nodeName = node.name()
        def isReserved = false
        def xmlComment = "'${nodeName}' is a reserved name, an underscore was appended to it..."

        if(reservedNames.contains(nodeName)) {
            isReserved = true
            nodeName +='_'
            println xmlComment
        }
        if(attrs) {
            builder."${nodeName}"(attrs) {
                if(isReserved) {
                    builder.mkp.comment(xmlComment)
                }
                node.attributes().each{attribute ->
                    builder."${attribute.key}"(attribute.value)
                }

                node.children().each { traverse(builder, it, null) }
            }
        } else {
            builder."${nodeName}" {
                if(isReserved) {
                    builder.mkp.comment(xmlComment)
                }
                node.attributes().each{attribute ->
                    builder."${attribute.key}"(attribute.value)
                }

                node.children().each { traverse(builder, it, null) }
            }
        }
    }
    /**
     * Output an XML file. An existing file will be deleted and recreated.
     * @param repository
     * @param repositoryAbsPath
     * @param outputFilePath
     * @param nodesToRemove
     * @param attributesToRemove
     */
    def static outputXML(repository, repositoryAbsPath, outputFilePath, nodesToRemove, attributesToRemove) {
        //println "$reservedNames"
        def session = MgnlContext.getHierarchyManager(repository).workspace.session
        def file = new File(outputFilePath)
        //recreate the file
        if(file.exists()) {
            file.delete()
            file.createNewFile()
        }
        def out = new FileWriter(file)
        def buffer = new ByteArrayOutputStream()
        nodesToRemove = nodesToRemove?:[]
        attributesToRemove = attributesToRemove?:[]

        def start = System.currentTimeMillis()

        session.exportDocumentView(repositoryAbsPath, buffer, false, false)

        def str = buffer.toString()

        def root = new XmlParser(false, false).parseText(str)

        root.depthFirst().each{node ->
            if(nodesToRemove.contains(node.name())){
                node.parent().remove(node)
            }
            node.attributes().each { entry ->
                if(entry.key.startsWith('xmlns:'))
                    attributesToRemove << entry.key
            }
            attributesToRemove.each {attr->
                node.attributes().remove(attr)
            }
        }

        def builder = new MarkupBuilder(out)
        builder.mkp.xmlDeclaration([version:"1.0",  encoding:"UTF-8"])
        traverse(builder,root, ["xmlns:jcr":"http://www.jcp.org/jcr/1.0"])

        //return "Export of workspace '${repository}' starting at '${repositoryAbsPath}' to '${file.path}' done in ${System.currentTimeMillis() - start} ms"
    }
}